import { Component, Input } from '@angular/core';
import {
  BackendService,
  ScrumTask,
  ScrumUserstory,
  ScrumStatus,
  ScrumCategory,
} from '../../../services/backend.service';
import { NgbModal } from '@ng-bootstrap/ng-bootstrap';
import { TableComponentBase } from '../table-component.base';
import { getNumberForPriority } from '../../../services/sorting.service';
import { UserstoryFormComponent } from '../../forms/userstory/userstory-form.component';
import { ActivatedRoute, ParamMap, Router } from '@angular/router';

@Component({
  selector: 'app-userstory-inner-table',
  templateUrl: './userstory-inner-table.component.html',
  styleUrls: ['./userstory-inner-table.component.css'],
})
export class UserstoryInnerTableComponent extends TableComponentBase<ScrumUserstory> {
  public tasks: ScrumTask[] = [];
  public filterPriority: string | null = null;
  public status: ScrumStatus[] = [];
  public categories: ScrumCategory[] = [];

  @Input() public items: ScrumUserstory[] = [];

  /**
   * Constructor that establishes the initial backend communication.
   * It also sets the tabel's items according to the "storys" input (needed for the dashboard).
   * @param backendService backendService object for backend communication
   * @param modalService angular modalService to handle modals for the Form popups
   * @param route route object to extract parameters from
   */
  constructor(
    private backendService: BackendService,
    private modalService: NgbModal,
    protected route: ActivatedRoute
  ) {
    super(route);

    backendService.getUserstories().subscribe((response) => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.items.push(...response.body);
      }
    });
    backendService.getTasks().subscribe((response) => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.tasks.push(...response.body);
      }
    });
    backendService.getAllStatus().subscribe((response) => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.status.push(...response.body);
      }
    });
    backendService.getCategories().subscribe((response) => {
      if (response.status > 399) {
        alert('Fehler');
      } else {
        this.categories.push(...response.body);
      }
    });
  }

  //#region getters

  /**
   * Returns a filtered list of all userstories that have a selected priority.
   * If no pritority to be filtered for is defined it just returns all userstories.
   */
  public get filteredItems() {
    if (this.filterPriority == null) {
      return this.items;
    }
    return this.items.filter((t) => t.priority.toLowerCase() == this.filterPriority.toLowerCase());
  }

  /**
   * Returns the number of related tasks of a given userstory.
   * The relation is defined by the userstoryId property of the task object and the id property of the userstory object.
   * @param userstory The userstory for which the number of related tasks gets calculated
   */
  public getNumberOfTasks(userstory: ScrumUserstory) {
    return this.tasks.filter((t) => t.userstoryId === userstory.id).length;
  }

  /**
   * Returns the title of the status that has the provided id.
   * @param id id of the searched status
   */
  getStatusTitleById(id) {
    var status = this.status.find((x) => x.id === id);
    if (!status) {
      return 'N/A';
    }
    return status.title;
  }

  /**
   * Returns the title of the category that has the provided id.
   * @param id id of the searched category
   */
  getCategoryTitleById(id) {
    var category = this.categories.find((x) => x.id === id);
    if (!category) {
      return 'N/A';
    }
    return category.title;
  }

  //#endregion getters

  //#region userstoryTableFunctions
  /**
   * Deletes a userstory by calling the delete method via the backendService and removing it from the tabel's items array.
   * @param userstory userstory that will be deleted
   */
  public deleteUserstory(userstory: ScrumUserstory) {
    this.backendService.deleteUserstory(userstory).subscribe((response) => {
      if (response.status > 399) {
        alert('Fehler');
      }
    });
    const index = this.items.indexOf(userstory);
    if (index !== -1) {
      this.items.splice(index, 1);
    }
  }

  //#endregion userstoryTableFunctions

  //#region sorters
  /**
   * Sorts the tabel's items by priority
   */
  public sortByPrio() {
    this.doNumericSort('priority', (us) => getNumberForPriority(us.priority));
  }

  /**
   * Sorts the tabel's items by amount of related tasks
   */
  public sortByTasks() {
    this.doNumericSort('tasks', (us) => this.getNumberOfTasks(us));
  }

  /**
   * Sorts the tabel's items by statusId
   */
  public sortByStatus() {
    this.doNumericSort('statusId', (us) => us.statusId);
  }

  /**
   * Sorts the tabel's items by categoryId
   */
  public sortByCategory() {
    this.doNumericSort('categoryId', (us) => us.categoryId);
  }

  //#endregion sorters

  //#region modals
  /**
   * Opens a UserstoryForm popup for editing a existing userstory or creating a new one.
   * @param editUserstory optional: userstory to edit (only needed if a userstory should be edited, not newly created)
   */
  public openUserstoryForm(editUserstory?: ScrumUserstory) {
    const modalRef = this.modalService.open(UserstoryFormComponent, {
      backdrop: 'static',
      keyboard: true,
      size: 'lg',
    });
    if (editUserstory == null) {
      modalRef.result.then((result) => {
        this.items.push(result);
      });
    }
    modalRef.componentInstance.userstory = editUserstory;
  }

  //#endregion modals
}
